/***************************************************************************
 *   Copyright (C) 2006 by Gerd v. Egidy                                   *
 *   gve@intra2net.com                                                     *
 *                                                                         *
 *   This library is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Lesser General Public License version   *
 *   2.1 as published by the Free Software Foundation.                     *
 *                                                                         *
 *   This library is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU Lesser General Public License for more details.                   *
 *                                                                         *
 *   You should have received a copy of the GNU Lesser General Public      *
 *   License along with this program; if not, write to the                 *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef __LIBT2N_SOCKET_SERVER
#define __LIBT2N_SOCKET_SERVER

#include <sys/types.h>
#include <string>

#include "server.hxx"
#include "socket_handler.hxx"
#include "types.hxx"

namespace libt2n
{

class socket_server_connection;

/** @brief Socket based server class

    Use this class to instantiate a server listening for client connections.
    Call fill_buffer() to read data from the network and get_packet() to retrieve
    this data. Don't forget to call cleanup() from time to time to remove closed
    connections and close idle ones.
*/
class socket_server : public socket_handler, public server
{
    friend class socket_server_connection;

    private:
        fd_set connection_set;
        std::string unix_path;

        void start_listening();

        void new_connection();

        bool fill_connection_buffers();
        void remove_connection_socket(int sock);

    protected:
        std::ostream* get_logstream(log_level_values level)
            { return server::get_logstream(level); }

    public:
        socket_server(int port, const std::string& ip="0.0.0.0");
        socket_server(const std::string& path, mode_t filemode=00770, const std::string& user="", const std::string& group="");

        ~socket_server();

        bool fill_buffer(long long usec_timeout=-1,long long* usec_timeout_remaining=NULL);
};

/** @brief Socket based connection

    This class is used within a socket_server to represent the connection to each client.
*/
class socket_server_connection : public socket_handler, public server_connection
{
    friend class socket_server;

    private:
        socket_server_connection(int _sock, socket_type_value _stype, int _timeout)
           : server_connection(_timeout), socket_handler(_sock,_stype)
           { }

        ~socket_server_connection();

        std::ostream* get_logstream(log_level_values level)
            { return server_connection::get_logstream(level); }

        void real_write(const std::string& data)
            { socket_write(data); }

    public:
        bool fill_buffer(long long usec_timeout=-1,long long* usec_timeout_remaining=NULL)
            { return socket_handler::fill_buffer(buffer,usec_timeout,usec_timeout_remaining); }

        virtual void close();
};

}

#endif
